## murahatebu : analyze_villagers.rb

require 'base.rb'


module MuraHatebu
  ## ǡɽ html ե
  ANALYZED_FILENAME = 'analyzed.html'

  ## ǡ¸ե
  ANALYZED_DATA_FILENAME = 'analyzed_data.dat'

  ## -- AnalyzedTemplate -----------------------------------------------
  class AnalyzedTemplate < HtmlTemplate
    FILENAME = Util.filename_to_path( 'template/analyzed_template.html' )
    PARAMS_STRUCT = Struct.new( :done_date, :version )

    OneRowTemplate = define_new_child( :count_order, [:villager, :analyzed_count, :list] )
    define_child( :ratio_order, OneRowTemplate )
    define_child( :both_order,  OneRowTemplate )
  end

  ## -- VillagerAnalyzedTemplate ---------------------------------------
  class VillagerAnalyzedTemplate < HtmlTemplate
    FILENAME = Util.filename_to_path( 'template/villager_analyzed_template.html' )
    PARAMS_STRUCT = Struct.new(
      :villager,
      :all_bookmark_count,
      :site,
      :favorite,
      :follower,
      :done_date,
      :analyzed_count,
      :no_comment_count,
      :comment_total_size,
      :version
    )
    BlackListTemplate = define_new_child( :black_list_count, [:filename, :count] )
    SiteTemplate = define_new_child( :site_count, [:site, :site_link, :count] )

    WeekTemplate         = define_new_child( :day_of_the_week_sort, [:height, :top, :value] )
    TimespanTemplate     = define_new_child( :timespan_sort, [:timespan, :width, :value] )
    WeekTimespanTemplate = define_new_child( :day_of_the_week_timespan_sort, [:timespan] )
    class WeekTimespanTemplate
      define_new_child( :day_of_the_week_sort, [:width, :value] )
    end
  end

  ## -- SiteAnalyzedTemplate -------------------------------------------
  class SiteAnalyzedTemplate < HtmlTemplate
    FILENAME = Util.filename_to_path( 'template/site_analyzed_template.html' )
    PARAMS_STRUCT = Struct.new( :site, :version )

    VillagerTemplate = define_new_child( :list, [:villager, :count] )
  end


  ## -- AnalyzedVillager -------------------------------------------------
  class AnalyzedVillager < Villager
    def initialize( name )
      super
      @analyzed_count = 0
      @black_list_analyzed_counts = Array.new( BlackList::KINDS.size, 0 )
      @no_comment_count = 0
      @comment_total_size = 0
      @site_count = {}
      @last_date_of_side_data_updated = Time.at( 0 ).to_date
      @week_timespan_sorter = WeekTimespanSorter.new
    end

    attr_accessor(
      :analyzed_count,
      :no_comment_count,
      :comment_total_size,
      :black_list_analyzed_counts,
      :site_count,
      :last_date_of_side_data_updated,
      :week_timespan_sorter
    )

    def get_black_list_analyzed_count_ratio( kind )
      return 0.0 if @analyzed_count == 0
      @black_list_analyzed_counts[kind].to_f / @analyzed_count.to_f
    end

    def add_site_count( site_url )
      @site_count[site_url] = 0 unless @site_count.has_key?( site_url )
      @site_count[site_url] += 1
    end
  end

  ## -- WeekTimespanSorter -----------------------------------------------
  class WeekTimespanSorter
    TIMESPAN_STRINGS = [
    "0   3 ",
    "3   6 ",
    "6   9 ",
    "9   12 ",
    "12   15 ",
    "15   18 ",
    "18   21 ",
    "21   0 ",
    ]

    def initialize
      @data = Array.new( Date::DAYNAMES.size ) { [0,0,0,0,0,0,0,0] }
    end

    attr_reader( :data )

    def hour_to_position( hour )
      return nil unless hour.kind_of?( Fixnum )
      return hour / 3
    end

    def add_count( time )
      @data[time.wday][self.hour_to_position( time.hour )] += 1
    end

    def week_each
      @data.each {|one|
        yield one.inject( 0 ) {|sum, one| sum + one }
      }
    end

    def timespan_each_with_index
      tmp = [0,0,0,0,0,0,0,0]
      @data.each {|one|
        tmp.each_index {|i|
          tmp[i] += one[i]
        }
      }
      tmp.each_with_index {|count, i| yield( count, i ) }
    end

    def each_with_timespan_index
      TIMESPAN_STRINGS.each_index {|i|
        k = -1
        tmp = Date::DAYNAMES.collect {
          k += 1
          @data[k][i]
        }
        yield( tmp, i )
      }
    end

    def total_count
      @data.inject( 0 ) {|sum, one| sum + one }
    end
  end

  ## -- Analyzer ---------------------------------------------------------
  class Analyzer
    ## -- ᥤ
    def start
      ## -- ֥åꥹȺ
      @black_list = MatchingList.new
      BlackList::FILENAME_TO_KIND_TABLE.each {|filename, kind|
        @black_list.read_file( filename ) { kind }
      }

      ## -- ǡѰ
      @database = PStore.new( Util.filename_to_path( ANALYZED_DATA_FILENAME ) )
      @done_date = {}
      @villagers = {}
      Util.load_village_people_file {|name|
        @villagers[name] = AnalyzedVillager.new( name )
      }
      @database.transaction {
        @done_date = @database.fetch( 'done_date', @done_date )
        if @database.root?( 'villagers' )
          old = @database['villagers']
          @villagers.each_key {|name|
            @villagers[name] = old[name] if old.has_key?( name )
          }
        end
      }
      @updated_site = {}

      unless @done_date.has_key?( YESTADAY )
        ## -- åɤ߹
        self.marge_daily_data( YESTADAY )

        ## -- դΥǡ
        @villagers.each_value{|villager|
          if villager.last_date_of_side_data_updated < TODAY - 3
            villager.get_side_data
            villager.last_date_of_side_data_updated = TODAY
          end
        }

        ## -- 礷ǡ¸
        @database.transaction {
          @database['villagers'] = @villagers
          @database['done_date'] = @done_date
        }
      end

      ## -- HTML 
      self.make_analyzed_html
      self.make_villager_html
      self.make_site_html
    end


    ## -- åɤ߹
    def marge_daily_data( date )
      daily_data = DailyData.new( date )
      unless daily_data.cache_exist?
        raise "cache file not exist : #{daily_data.cache_filepath}"
      end
      daily_data.load_cache

      @done_date[date] = true

      daily_data.data.each {|bookmark|
        next unless @villagers.has_key?( bookmark.user.name )
        villager = @villagers[bookmark.user.name]

        villager.analyzed_count += 1
        black_list_kind = @black_list.match( bookmark.target_url )
        villager.black_list_analyzed_counts[black_list_kind] += 1 if black_list_kind
        villager.no_comment_count += 1 if bookmark.comment.to_s.empty?
        bookmark.tags.each {|tag| villager.comment_total_size += tag.size }
        villager.comment_total_size += bookmark.comment.to_s.size

        villager.week_timespan_sorter.add_count( bookmark.time )

        site_url = Util.get_site_url_from_url( bookmark.target_url )
        villager.add_site_count( site_url )
        @updated_site[site_url] = true
      }
    end


    ## -- HTML Ϸ -----
    ## -- ״֤ʸ
    def make_done_date_str
      date_group = [[]]
      @done_date.keys.sort.each {|date|
        date_group << [] if ( not date_group.last.empty? ) && date_group.last.last != date - 1
        date_group.last << date
      }
      date_group.collect {|ary|
        ' ' << Util.enc( ary.first.to_japanese_string ) <<
        ( ary.size == 1 ? '' : Util.enc( '' + ary.last.to_japanese_string ) )
      }.join( ', ' )
    end

    ## -- ᥤ HTML 
    def make_analyzed_html
      analyzed_template = AnalyzedTemplate.new

      ## ֥޿󥭥
      count_order = @villagers.values.sort_by {|v|
        [v.black_list_analyzed_counts[BlackList::Kind::TowChannel], -1 * v.analyzed_count]
      }.reverse
      ## ֥޳󥭥
      ratio_order = @villagers.values.sort_by {|v|
        [v.get_black_list_analyzed_count_ratio( BlackList::Kind::TowChannel ), v.analyzed_count]
      }.reverse

      ## ¼
      make_one_row = lambda {|villager, one_row_template|
        one_row_template.params.villager = villager.name
        one_row_template.params.analyzed_count = villager.analyzed_count
        one_row_template.params.list = BlackList::KINDS.collect {|kind|
          Tag.td { villager.black_list_analyzed_counts[kind].to_s } <<
          Tag.td { format( "%.4f", villager.get_black_list_analyzed_count_ratio( kind ) ) }
        }.join
      }

      count_order.each {|villager|
        analyzed_template.params.count_order {|template| make_one_row.call( villager, template ) }
      }
      ratio_order.each {|villager|
        analyzed_template.params.ratio_order {|template| make_one_row.call( villager, template ) }
      }
      (ratio_order[0, 30] & count_order[0, 30]).each {|villager|
        analyzed_template.params.both_order {|template| make_one_row.call( villager, template ) }
      }

      analyzed_template.params.done_date = self.make_done_date_str
      analyzed_template.params.version = VERSION
      analyzed_template.write_to_file( Util.filename_to_path( ANALYZED_FILENAME ) )
    end

    ## -- ¼̲ HTML 
    def make_villager_html
      villager_template = VillagerAnalyzedTemplate.new

      count_ratio_to_s = lambda {|count, total|
        tmp = total.zero? ? 0.0 : (count.to_f / total.to_f)
        "#{count} (#{format( '%.4f', tmp )})"
      }

      @villagers.each_value{|villager|
        villager_template.reset

        BlackList::KINDS.each {|kind|
          villager_template.params.black_list_count {|black_list_template|
            black_list_template.params.filename = BlackList::KIND_TO_FILENAME_TABLE[kind]
            black_list_template.params.count =
            count_ratio_to_s.call( villager.black_list_analyzed_counts[kind], villager.analyzed_count )
          }
        }
        villager.site_count.to_a.sort_by {|site, count| count
        }.reverse_each {|site, count|
          villager_template.params.site_count {|site_template|
            site_template.params.site = site
            site_template.params.site_link = site.gsub( /\//, '.' )
            site_template.params.count = count.to_s
          }
        }

        villager.week_timespan_sorter.week_each {|week_total|
          villager_template.params.day_of_the_week_sort {|week_template|
            tmp = villager.analyzed_count.zero? ? 0 : 500 * ( week_total.to_f / villager.analyzed_count )
            week_template.params.height = tmp.to_i
            week_template.params.top    = tmp.to_i - 20
            week_template.params.value  = week_total
          }
        }

        villager.week_timespan_sorter.timespan_each_with_index {|timespan_total, i|
          villager_template.params.timespan_sort {|timespan_template|
            timespan_template.params.timespan = Util.enc( WeekTimespanSorter::TIMESPAN_STRINGS[i] )
            timespan_template.params.width = villager.analyzed_count.zero? ? 0 : (500 * ( timespan_total.to_f / villager.analyzed_count )).to_i
            timespan_template.params.value = timespan_total
          }
        }

        villager.week_timespan_sorter.each_with_timespan_index {|week_ary, i|
          villager_template.params.day_of_the_week_timespan_sort{|timespan_template|
            timespan_template.params.timespan = Util.enc( WeekTimespanSorter::TIMESPAN_STRINGS[i] )
            week_ary.each {|count|
              timespan_template.params.day_of_the_week_sort {|week_template|
                week_template.params.width = villager.analyzed_count.zero? ? 0 : (800 * ( count.to_f / villager.analyzed_count )).to_i
                week_template.params.value = count
              }
            }
          }
        }

        villager_template.params.no_comment_count =
        count_ratio_to_s.call( villager.no_comment_count, villager.analyzed_count )

        villager_template.params.comment_total_size =
        count_ratio_to_s.call( villager.comment_total_size, villager.analyzed_count )

        villager_template.params.villager = villager.name
        villager_template.params.all_bookmark_count = villager.bookmark_count
        villager_template.params.site = villager.site.nil? ? '---' : Tag.a( villager.site ) { villager.site }
        villager_template.params.favorite = villager.favorite_count.to_s
        villager_template.params.follower = villager.follower_count.to_s
        villager_template.params.done_date = self.make_done_date_str
        villager_template.params.analyzed_count = villager.analyzed_count
        villager_template.params.version = VERSION
        villager_template.write_to_file( "villager_analyzed/#{villager.name}.html" )
      }
    end

    ## -- ̲ HTML 
    def make_site_html
      ## -- ǡ
      site_hash = Hash.new{|h, site| h[site] = {} }
      @villagers.each_value{|villager|
        villager.site_count.each{|site, count|
          site_hash[site][villager.name] = count
        }
      }

      site_template = SiteAnalyzedTemplate.new

      @updated_site.each_key {|site|
        site_template.reset
        site_template.params.site = site
        site_hash[site].to_a.sort_by {|villager, count| count
        }.reverse_each {|name, count|
          site_template.params.list {|villager_template|
            villager_template.params.villager = name
            villager_template.params.count = count.to_s
          }
          site_template.params.version = VERSION
          site_template.write_to_file( "site_analyzed/#{site.to_s.gsub( /\//, '.' )}.html" )
        }
      }
    end

  end
end
## -- main ---------------------------------------------------------------
MuraHatebu::TODAY = Date.parse( ARGV[0] ) rescue Date.today
MuraHatebu::YESTADAY = MuraHatebu::TODAY - 1
MuraHatebu::Analyzer.new.start
