## murahatebu : base.rb

require 'time'
require 'date'
require 'pstore'
require 'cgi'
begin
  require 'cgi/html'
rescue LoadError
end

require 'hatebu.rb'


class Date
  def to_continuous_string
    self.strftime( '%Y%m%d' )
  end

  def to_standard_string
    self.strftime( '%Y/%m/%d' )
  end

  def to_japanese_string
    self.strftime( '%Yǯ%m%d' )
  end

  def same_month?( other )
    self.year == other.year && self.month == other.month
  end
end

class Time
  def to_date
    Date.new( self.year, self.month, self.day )
  end

  def to_standard_string
    self.strftime( '%Y/%m/%d %H:%M:%S' )
  end
end


module MuraHatebu
  VERSION = '1.0.0'

  ## 顼ե
  ERROR_LOG_FILENAME = 'error.log'

  ## ¼ãꥹȥե
  VILLAGE_PEOPLE_FILENAME = 'village_people'

  ## -- ¼Ϥƥַϥǡ¸Υ桼ƥƥؿ
  Util = Hatebu::Util
  module Util
    def self.filename_to_path( filename )
      File.dirname( File.expand_path( __FILE__ ) ) + '/' + filename
    end


    def self.error_log
      File.open( Util.filename_to_path( ERROR_LOG_FILENAME ), "a" ) {|file|
        class << file
          alias :murahatebu_write :write
          def write( str )
            self.murahatebu_write( str )
            $stdout.write( str )
          end
        end
        yield file
      }
    end

    def self.default_error_log( exception, addition )
      self.error_log {|file|
        file.puts "====== ERROR : #{addition} : #{Time.now.to_standard_string} ====="
        file.puts exception.inspect
        file.puts exception.backtrace
      }
    end

    def self.loadfile_as_standard_format( path )
      File.foreach( path ) {|line|
        line.strip!
        next if /^\#\#/.match( Util.enc( line ) )
        next if line.empty?
        yield line
      }
    end

    def self.file_puts( filepath, mode, data )
      File.open( filepath, mode ) {|file|
        file.puts( data )
      }
    end

    SORT_OUT_SITE = {}
    [
    'blog.livedoor.jp',
    'd.hatena.ne.jp',
    'twitter.com',
    'ameblo.jp',
    ].each {|one| SORT_OUT_SITE[one] = true }
    def self.get_site_url_from_url( url )
      begin
        uri = URI.parse( url )
        _site = uri.host
        if SORT_OUT_SITE.has_key?( _site )
          _site += ( uri.path.sub( /(\/[^\/]*)\/.*$/, "\\1" ) )
        elsif Regexp.compile( Regexp.escape( "http://b.hatena.ne.jp/articles/" ) ).match( @url )
          _site = 'b.hatena.ne.jp/articles'
        end
        _site
      rescue
        /https?:\/\/([^\/]*)\//.match( url )
        return $1.nil? ? '' : $1
      end
    end

    def self.make_html_linked_title( url, title )
      Tag.a( url ) { CGI.escapeHTML( title ) } + self.make_linked_bookmark_image( url )
    end

    def self.make_linked_bookmark_image( url )
      Tag.a( Util.make_bookmark_url( url ) ) {
        Tag.img( Util.make_bookmark_image_url( url ) )
      }
    end

    def self.date_to_daily_filename( date, type )
      "archives/#{date.to_continuous_string}_#{type}.html"
    end

    def self.load_village_people_file
      Util.loadfile_as_standard_format( Util.filename_to_path( VILLAGE_PEOPLE_FILENAME ) ) {|name|
        yield name
      }
    end
  end

  ## 饹
  Tag = Object.new
  Tag.extend( CGI::Html4 )
  Tag.element_init()
  Tag.extend( CGI::HtmlExtension )

  ## -- MatchingList -----------------------------------------------------
  class MatchingList
    def initialize( path = nil )
      @root = {}
      @files = []
      self.read_file( path ) {|one| block_given? ? (yield one) : path } if path
    end

    attr_reader( :files )

    def read_file( path )
      @files << path
      Util.loadfile_as_standard_format( path ) {|str|
        self.add( str ) {|one| block_given? ? (yield one) : path }
      }
    end

    def add( str )
      current = @root
      str.each_byte {|c|
        current[c] = {} unless current.has_key?( c )
        current = current[c]
      }
      current[nil] = block_given? ? (yield str) : true
    end

    def match( str )
      current = @root
      str.each_byte {|c|
        return false unless current.has_key?( c )
        current = current[c]
        return current[nil] if current.has_key?( nil )
      }
      current[nil]
    end
  end

  ## -- HtmlTemplate -----------------------------------------------------
  class HtmlTemplate
    ## FILENAME = <filepath>
    ## PARAMS_STRUCT = Struct.new( <members ...> )
    def self.define_child( parameter_name, klass )
      self.const_set( :CHILDREN, {} ) unless self.const_defined?( :CHILDREN )
      self::CHILDREN[parameter_name] = klass

      self::PARAMS_STRUCT.class_eval {
        define_method( parameter_name ) {|&block|
          @__template.child_make_html( parameter_name, block )
        }
      }
    end

    def self.define_new_child( parameter_name, child_parameters )
      klass = Class.new( HtmlTemplate )
      klass.const_set( :PARAMS_STRUCT, Struct.new( *child_parameters ) )
      self.define_child( parameter_name, klass )
      klass
    end

    def initialize
      @params = (self.class)::PARAMS_STRUCT.new
      @params.instance_variable_set( :@__template, self )
      @children_result = {}
      @children_template = {}

      @source = block_given? ? yield : Util.enc( File.read( (self.class)::FILENAME ) )

      tmp = @source
      loop {
        m = /\%\%([a-zA-Z_][0-9A-Za-z_]*)\%\%\{(.*?)\}\%\%\1\%\%/m.match( tmp )
        break if m.nil?
        name = m[1].to_sym
        @children_template[name] = (self.class)::CHILDREN[name].new { m[2] }
        tmp = "#{m.pre_match}@@#{name}@@#{m.post_match}"
      }
      @base = tmp
      self.reset
    end

    def reset
      @params.members.each {|one| @params[one] = '' }
      @children_template.each_key{|key| @children_result[key] = '' }
    end

    def make_html
      result = @base.dup
      @params.each_pair {|param, value|
        result.gsub!( Regexp.compile( Regexp.escape( "@@#{param}@@" ) ) ) { value.to_s }
      }
      @children_result.each_pair {|param, value|
        result.gsub!( Regexp.compile( Regexp.escape( "@@#{param}@@" ) ) ) { value.to_s }
      }
      result
    end

    attr_accessor( :params )
    attr_reader( :children_result )

    def child_make_html( parameter_name, block )
      template = @children_template[parameter_name]
      template.reset
      block.call( template )
      @children_result[parameter_name] << template.make_html
    end

    def to_s
      self.make_html
    end

    def write_to_file( path )
      Util.file_puts( path, "w", self.make_html )
    end
  end

  ## -- Villager ---------------------------------------------------------
  Villager = Hatebu::User
  class Villager
    alias :initialize_original :initialize
    def initialize( name )
      initialize_original( name )
      @bookmarks = []
    end

    attr_reader( :bookmarks )

    def sort_bookmarks_by_time
      @bookmarks = @bookmarks.sort_by {|bookmark| bookmark.time }
    end
  end

  ## -- Bookmark ---------------------------------------------------------
  Bookmark = Hatebu::Bookmark

  ## -- Entry ------------------------------------------------------------
  Entry = Hatebu::Entry
  class Entry
    TITLE_CONVERT_LIST = MatchingList.new
    [
    ['http://anond.hatelabo.jp/', ''],
    ['http://kyoko-np.net/',      'ʹ'],
    ['http://jp.techcrunch.com/', 'TechCrunch'],
    ['http://alfalfalfa.com/',    'եե⥶'],
    ['http://digimaga.net/',      'ǥޥ'],
    ].each {|url, site_name|
      TITLE_CONVERT_LIST.add( url ) { site_name }
    }

    def set_title( title )
      @title_original = title
      site_name = TITLE_CONVERT_LIST.match( @target_url )
      @title = site_name ? "#{Util.enc( site_name )} : #{title}" : title
    end
    attr_reader( :title_original )

    alias :get_data_original :get_data
    def get_data
      self.get_data_original
      self.set_title( @title )
    end

    def make_html_linked_title
      Util.make_html_linked_title( @target_url, @title )
    end

    def sort_bookmarks_by_time
      @bookmarks = @bookmarks.sort_by {|bookmark| bookmark.time }
    end
  end

  ## -- DailyData --------------------------------------------------------
  class DailyData
    def initialize( date )
      @date = date
      @cache_filepath = 'cache/' << @date.to_continuous_string << '.dat'
      @data = []
    end

    attr_reader( :date, :cache_filepath, :data )

    def cache_exist?
      File.exists?( @cache_filepath )
    end

    def load_cache
      PStore.new( @cache_filepath ).transaction {|db|
        @data = db['data']
      }
    end

    def save_cache
      PStore.new( @cache_filepath ).transaction {|db|
        db['data'] = @data
      }
    end

    def get_data( villagers )
      @data = []
      villagers.each {|name, villager|
        begin
          villager.get_bookmark {|bookmark|
            next if bookmark.time.to_date > @date
            @data << bookmark if bookmark.time.to_date == @date
            Villager::STOP_GET_BOOKMARK if bookmark.time.to_date < @date
          }
        rescue => e
          Util.default_error_log( e, "villager = #{name} : #{__LINE__}" )
        end
      }
    end
  end

  ## -- CollectedDailyData -----------------------------------------------
  class CollectedDailyData
    def initialize( villagers )
      @villager_hash = {}
      villagers.each {|name, villager|
        @villager_hash[name] = Villager.new( name )
      }
      @entry_hash = Hash.new {|hash, url| hash[url] = Entry.new( url ) }
    end

    attr_reader( :villager_hash, :entry_hash )

    def add_daily_data( daily_data )
      updated_entry = {}
      updated_villager = {}
      daily_data.data.each {|bookmark|
        entry = @entry_hash[bookmark.target_url]
        entry.bookmarks << bookmark
        updated_entry[entry] = true
        if @villager_hash.has_key?( bookmark.user.name )
          villager = @villager_hash[bookmark.user.name]
          villager.bookmarks << bookmark
          updated_villager[villager] = true
        end
      }
      updated_entry.each_key {|entry|
        entry.sort_bookmarks_by_time
        entry.set_title( entry.bookmarks.last.title )
      }
      updated_villager.each_key {|villager| villager.sort_bookmarks_by_time }
    end
  end


  ## -- ֥åꥹȴϢ
  module BlackList
    module FileName
      TowChannel = 'url_black_list_2ch'
      Ero        = 'url_black_list_ero'
      Spam       = 'url_black_list_spam'
      KusoMedia  = 'url_black_list_kuso_media'
    end

    module Kind
      TowChannel = 0
      Ero        = 1
      Spam       = 2
      KusoMedia  = 3
    end

    KINDS = [
    Kind::TowChannel,
    Kind::Ero,
    Kind::Spam,
    Kind::KusoMedia,
    ]

    FILENAMES = [
    FileName::TowChannel,
    FileName::Ero,
    FileName::Spam,
    FileName::KusoMedia,
    ]

    KIND_TO_FILENAME_TABLE = FILENAMES

    FILENAME_TO_KIND_TABLE = {}
    KIND_TO_FILENAME_TABLE.each_with_index {|filename, index|
      FILENAME_TO_KIND_TABLE[filename] = index
    }
  end
end
