## murahatebu : murahatebu.rb

require 'base.rb'


## ǽ
MuraHatebu::NEW_ENTRY_COLLECT_DAY = 2

## ɽ
MuraHatebu::NEW_ENTRY_THRESHOLD = 6

## ͵ǽ
MuraHatebu::HOT_ENTRY_COLLECT_DAY = 3

## ͵ɽ
MuraHatebu::HOT_ENTRY_DISPLAY_COUNT = 100

## ꤹ¼ͤοͿ
MuraHatebu::CHOSEN_ENTRY_VILLAGERS_THRESHOLD = 10

## ꤹΥݥȤ
MuraHatebu::CHOSEN_ENTRY_POINT_THRESHOLD = 30

## ȥåץڡΥե̾
MuraHatebu::INDEX_FILENAME = 'index.shtml'

## ꤹΥե̾
MuraHatebu::CHOSEN_FILENAME = 'chosen.html'

## ꤹΥե̾
MuraHatebu::CHOSEN_YESTADAY_FILENAME = 'chosen_before_yestaday.html'

## ԤФ뤿Υȥ̤
MuraHatebu::CANDIDATE_HOT_ENTRY_COUNT_THRESHOLD = 100

## ԤΥե̾
MuraHatebu::CANDIDATE_FILENAME = 'candidate.html'

## Ԥʤꥹ
MuraHatebu::MURAHACHIBU_LIST_FILENAME = 'murahachibu'



module MuraHatebu
  ## -- DailyTemplate ----------------------------------------------------
  class DailyTemplate < HtmlTemplate
    FILENAME = Util.filename_to_path( 'template/daily_template.html' )
    PARAMS_STRUCT = Struct.new( :date, :type, :version )

    OneRowTemplate = define_new_child( :list, [:number, :linked_title, :point, :villager_count] )
    class OneRowTemplate < HtmlTemplate
      FaviconTemplate = define_new_child( :favicons, [:villager, :tags, :comment] )
    end
  end

  ## -- IndexTemplate ----------------------------------------------------
  class IndexTemplate < HtmlTemplate
    FILENAME = Util.filename_to_path( 'template/index_template.html' )
    PARAMS_STRUCT = Struct.new( :top_three, :yestaday, :before_yestaday, :version )

    RecentTemplate   = define_new_child( :recent, [:japanese, :continuous] )
    CalendarTemplate = define_new_child( :calendar, [:year, :month, :row_count, :days_table] )
  end

  ## -- CandidateTemplate ------------------------------------------------
  class CandidateTemplate < HtmlTemplate
    FILENAME = Util.filename_to_path( 'template/candidate_template.html' )
    PARAMS_STRUCT = Struct.new( :list, :version )
  end

  ## -- RSSTemplate ------------------------------------------------------
  class RSSTemplate < HtmlTemplate
    FILENAME = Util.filename_to_path( 'template/rss_template.txt' )
    PARAMS_STRUCT = Struct.new( :type, :rss_filename, :entry_filename )

    SequenceTemplate = define_new_child( :sequences, [:url] )
    ItemTemplate     = define_new_child( :items, [:url, :title, :time, :bookmark_url, :bookmark_image_url] )
    class ItemTemplate
      IconTemplate    = define_new_child( :icons, [:villager] )
      CommentTemplate = define_new_child( :comments, [:villager, :permalink_url, :tags, :comment, :date] )
    end

    def initialize
      super
      @entry_array = []
    end

    attr_reader( :entry_array )

    def output_as_chosen
      self.params.type = Util.enc( 'ꤹ' )

      self.params.rss_filename = 'rss/chosen.xml'
      self.set_params_of_entry_array( false )
      self.write_to_file( Util.filename_to_path( self.params.rss_filename ) )

      self.params.rss_filename = 'rss/chosen_with_comment.xml'
      self.set_params_of_entry_array( true )
      self.write_to_file( Util.filename_to_path( self.params.rss_filename ) )
    end

    def output_as_new
      self.params.type = Util.enc( '' )
      self.params.rss_filename = 'rss/new.xml'
      self.set_params_of_entry_array( false )
      self.write_to_file( Util.filename_to_path( self.params.rss_filename ) )

      self.params.rss_filename = 'rss/new_with_comment.xml'
      self.set_params_of_entry_array( true )
      self.write_to_file( Util.filename_to_path( self.params.rss_filename ) )
    end

    def output_as_hot
      self.params.type = Util.enc( '͵' )
      self.params.rss_filename = format( 'rss/hot_%03d.xml', @entry_array.size )
      self.set_params_of_entry_array( false )
      self.write_to_file( Util.filename_to_path( self.params.rss_filename ) )

      self.params.rss_filename = format( 'rss/hot_%03d_with_comment.xml', @entry_array.size )
      self.set_params_of_entry_array( true )
      self.write_to_file( Util.filename_to_path( self.params.rss_filename ) )
    end

    def set_params_of_entry_array( with_comment )
      now_string = Time.now.strftime( '%Y-%m-%dT%H:%M:%S+09:00' )
      @children_result[:sequences] = ''
      @children_result[:items] = ''
      @entry_array.each {|entry|
        self.params.sequences {|sequences_template|
          sequences_template.params.url = CGI.escapeHTML( entry.target_url )
        }
        self.params.items {|item_template|
          item_template.params.url = CGI.escapeHTML( entry.target_url )
          item_template.params.title = CGI.escapeHTML( entry.title )
          item_template.params.time = now_string
          item_template.params.bookmark_url = Util.make_bookmark_url( entry.target_url )
          item_template.params.bookmark_image_url = Util.make_bookmark_image_url( entry.target_url )
          entry.bookmarks.each {|bookmark|
            unless with_comment
              item_template.params.icons {|icon_template|
                icon_template.params.villager = bookmark.user.name
              }
            else
              item_template.params.comments {|comment_template|
                comment_template.params.villager = bookmark.user.name
                comment_template.params.permalink_url = bookmark.permalink_url
                comment_template.params.tags = bookmark.tags.join( ' ' )
                comment_template.params.comment = CGI.escapeHTML( bookmark.comment.to_s )
              }
            end
          }
        }
      }
    end
  end

  ## -- CandidateData ----------------------------------------------------
  class CandidateData < Hash

    class Candidate < Villager
      def initialize( name )
        super( name )
        @count = 0
        @comment_total_size = 0
        @last_date_of_side_data_updated = Time.at( 0 ).to_date
      end
      attr_accessor( :count, :comment_total_size, :last_date_of_side_data_updated )
      def comment_size_ratio
        return 0.0 if @count == 0
        @comment_total_size.to_f / @count.to_f
      end
    end

    def initialize
      @done_date = {}
    end

    attr_reader( :done_date )

    def []( name )
      self[name] = Candidate.new( name ) unless self.has_key?( name )
      super( name )
    end

    def collect_data_from_url( url )
      entry = Entry.new( url )
      begin
        entry.get_data
        entry.bookmarks.each{|bookmark|
          candidate = self[bookmark.user.name]
          candidate.count += 1
          bookmark.tags.each{|tag| candidate.comment_total_size += tag.size }
          candidate.comment_total_size += bookmark.comment.size
        }
      rescue => e
        Util.default_error_log( e, "url = #{url} : #{__LINE__}" )
      end
    end
  end


  ## -- Manager ----------------------------------------------------------
  class Manager

    ## -- HTML  ----------------------------------------------------
    ## 1 ʬ
    def make_one_row_html( one_row_template, number, entry, point )
      one_row_template.params.number = number
      one_row_template.params.linked_title = entry.make_html_linked_title
      one_row_template.params.point = point
      one_row_template.params.villager_count = entry.bookmarks.size
      entry.bookmarks.each{|bookmark|
        one_row_template.params.favicons {|favicon_template|
          favicon_template.params.villager = bookmark.user.name
          favicon_template.params.tags = bookmark.tags.join( ' ' )
          favicon_template.params.comment = bookmark.comment.to_s.gsub( /\'/, "\\\\'" ).gsub( /\"/, '&quot;' )
        }
      }
    end

    ## --  1 ʬ html ե
    def make_new_html
      entry_count = 0
      @daily_template.reset
      @rss_template.entry_array.clear

      @collected_daily_data.entry_hash.values.sort_by{|entry| entry.bookmarks.first.time
      }.reverse_each{|entry|
        new_point = self.get_new_point( entry )
        next if new_point < NEW_ENTRY_THRESHOLD

        @daily_template.params.list {|one_row_template|
          self.make_one_row_html( one_row_template, entry_count + 1, entry, new_point )
        }

        @rss_template.entry_array << entry

        entry_count += 1
      }

      ## -- ƥץ졼Ȥ
      @daily_template.params.date = Util.enc( YESTADAY.to_japanese_string )
      @daily_template.params.type = Util.enc( '' )
      @daily_template.params.version = VERSION
      filepath = Util.filename_to_path( "archives/#{YESTADAY.to_continuous_string}_new.html" )
      @daily_template.write_to_file( filepath )

      ## -- RSS 
      @rss_template.params.entry_filename = filepath
      @rss_template.output_as_new
    end

    ## -- ͵ 1 ʬ html ե
    def make_hot_html
      entry_count = 0
      @daily_template.reset
      @rss_template.entry_array.clear
      filepath = Util.filename_to_path( "archives/#{YESTADAY.to_continuous_string}_hot.html" )
      @rss_template.params.entry_filename = filepath

      @collected_daily_data.entry_hash.values.sort_by{|entry| [self.get_hot_point( entry ), entry.bookmarks.first.time]
      }.reverse_each{|entry|
        @daily_template.params.list {|one_row_template|
          self.make_one_row_html( one_row_template, entry_count + 1, entry, self.get_hot_point( entry ) )
        }
        entry_count += 1

        unless @candidate_data.done_date.has_key?( YESTADAY )
          @candidate_data.collect_data_from_url( entry.target_url ) if entry_count <= CANDIDATE_HOT_ENTRY_COUNT_THRESHOLD
        end

        if entry_count == 3
          @top_three_html = @daily_template.children_result[:list].dup
          @top_three_html.gsub!( /\"\.\.\//, "\"./" )
        end

        ## -- RSS 
        @rss_template.entry_array << entry
        @rss_template.output_as_hot if entry_count % 5 == 0

        break if entry_count >= HOT_ENTRY_DISPLAY_COUNT
      }
      @candidate_data.done_date[YESTADAY] = true

      ## -- ƥץ졼Ȥ
      @daily_template.params.date = Util.enc( YESTADAY.to_japanese_string )
      @daily_template.params.type = Util.enc( '͵' )
      @daily_template.params.version = VERSION
      @daily_template.write_to_file( filepath )
    end

    ## -- ꤹ html ե
    def make_chosen_html
      entry_count = 0
      @daily_template.reset
      @rss_template.entry_array.clear

      @database.transaction {
        @database['chosen_yestaday_copy'] = {} unless @database.root?( 'chosen_yestaday_copy' )
        unless @database['chosen_yestaday_copy'].has_key?( YESTADAY )
          if File.exist?( Util.filename_to_path( CHOSEN_FILENAME ) )
            File.rename( Util.filename_to_path( CHOSEN_FILENAME ), Util.filename_to_path( CHOSEN_YESTADAY_FILENAME ) )
          end
          @database['chosen_yestaday_copy'][YESTADAY] = true
        end
      }

      @collected_daily_data.entry_hash.values.sort_by{|entry| [self.get_chosen_point( entry ), entry.bookmarks.first.time]
      }.reverse_each{|entry|
        chosen_point = self.get_chosen_point( entry )
        next if chosen_point < CHOSEN_ENTRY_POINT_THRESHOLD

        @daily_template.params.list {|one_row_template|
          self.make_one_row_html( one_row_template, entry_count + 1, entry, chosen_point )
        }
        @rss_template.entry_array << entry

        entry_count += 1
      }

      ## -- ƥץ졼Ȥ
      @daily_template.params.date = Util.enc( YESTADAY.to_japanese_string )
      @daily_template.params.type = Util.enc( 'ꤹ' )
      @daily_template.params.version = VERSION
      daily_html = @daily_template.make_html
      daily_html.gsub!( /\"\.\.\//, "\"./" )
      Util.file_puts( Util.filename_to_path( CHOSEN_FILENAME ), "w", daily_html )

      ## -- RSS 
      @rss_template.params.entry_filename = CHOSEN_FILENAME
      @rss_template.output_as_chosen
    end

    ## -- @candidate_data  html ˽
    def make_candidate_html
      template = CandidateTemplate.new

      exclude = {}
      @villagers.each_key{|name| exclude[name] = true }
      Util.loadfile_as_standard_format( Util.filename_to_path( MURAHACHIBU_LIST_FILENAME ) ) {|name|
        exclude[name] = true
      }

      candidate_array = @candidate_data.to_a.select{|name, candidate| not exclude[name] }
      count_top = candidate_array.sort_by{|name, candidate| candidate.count }.last( 100 ).reverse
      comment_top = candidate_array.sort_by{|name, candidate| candidate.comment_total_size }.last( 100 ).reverse

      (count_top + comment_top).each{|name, candidate|
        if candidate.last_date_of_side_data_updated < TODAY - 3
          candidate.get_side_data
          candidate.last_date_of_side_data_updated = TODAY
        end
      }

      one_data_proc = lambda {|name, candidate|
        Tag.td{ Tag.a( "http://b.hatena.ne.jp/" + name ) { name } } <<
        Tag.td{ candidate.count } <<
        Tag.td{ candidate.comment_total_size } <<
        Tag.td{ format( "%.3f", candidate.comment_size_ratio ) } <<
        Tag.td{ candidate.site.nil? ? '---' : Tag.a( candidate.site ) { Util.enc( '' ) } } <<
        Tag.td{ candidate.favorite_count } <<
        Tag.td{ candidate.follower_count }
      }
      100.times{|i|
        template.params.list << Tag.tr {
          one_data_proc.call( *count_top[i] ) <<
          one_data_proc.call( *comment_top[i] ) << "\n"
        }
      }
      template.params.version = VERSION
      template.write_to_file( Util.filename_to_path( CANDIDATE_FILENAME ) )
    end

    ## -- index.shtml κ
    def make_index_html
      index_template = IndexTemplate.new
      day_hash = {}

      ## -- Υեθ
      Dir.glob( Util.filename_to_path( 'archives/*.html' ) ) {|filepath|
        m = /archives\/(20([1-3][0-9])(0[1-9]|1[0-2])(0[1-9]|[12][0-9]|3[01]))_.*\.html/.match( filepath )
        begin
          day_hash[Time.parse( m[1] ).to_date] = true if m
        rescue => e
          Util.default_error_log( e, "file = #{filepath} : #{__LINE__}" )
        end
      }

      ## -- ǶΥꥹ
      day_hash.keys.sort.last( 10 ).reverse_each{|date|
        index_template.params.recent {|recent_template|
          recent_template.params.japanese = Util.enc( date.to_japanese_string )
          recent_template.params.continuous = date.to_continuous_string
        }
      }

      ## -- Υ
      month_hash = {}
      day_hash.each_key{|date|
        month_hash[Date.new( date.year, date.month, 1 )] = true
      }
      month_hash.keys.sort.reverse_each{|first_day|
        index_template.params.calendar{|calendar_template|
          calendar_template.set_data_for_making( first_day, day_hash )
          calendar_template.make_table
        }
      }

      index_template.params.top_three = @top_three_html
      index_template.params.yestaday = Util.enc( YESTADAY.to_japanese_string )
      index_template.params.before_yestaday = Util.enc( (YESTADAY - 1).to_japanese_string )
      index_template.params.version  = VERSION
      index_template.write_to_file( Util.filename_to_path( INDEX_FILENAME ) )
    end

    ## -- ɽ
    ## -- CalendarTemplate -------------------------------------------------
    class ::MuraHatebu::IndexTemplate::CalendarTemplate
      def set_data_for_making( date, day_hash )
        @params.year = date.year.to_s
        @params.month = date.month.to_s
        @month_first = Date.new( date.year, date.month, 1 )
        @month_last = (@month_first >> 1) - 1
        @day_hash = day_hash
      end

      def each
        @month_first.wday.times{ yield nil }
        @month_first.step( @month_last ) {|date|
          yield date
        }
        (6 - @month_last.wday).times{ yield nil }
      end

      def each_week
        ary = []
        self.each{|date|
          ary << date
          if ary.size >= 7
            yield ary
            ary = []
          end
        }
      end

      def day_to_html( date, type )
        return  '' if date.nil?
        tmp = date.day.to_s
        tmp = Tag.a( Util.date_to_daily_filename( date, type ) ) { tmp } if @day_hash.has_key?( date )
        tmp
      end

      def make_table
        week_count = 0
        self.each_week{|week|
          row_new = ''
          row_hot = ''
          week_count += 1
          week.collect{|date|
            row_new << Tag.td { self.day_to_html( date, 'new' ) }
            row_hot << Tag.td { self.day_to_html( date, 'hot' ) }
          }
          @params.days_table << Tag.tr { row_new + row_hot } << "\n"
        }
        @params.row_count = (week_count + 2).to_s
      end
    end


    ## -- ᥤν -------------------------------------------------
    ## -- ƼݥȤλХ᥽å
    def get_new_point( entry )
      result = 0
      result -= 7 if @black_list.match( entry.target_url )
      entry.bookmarks.each{|bookmark|
        case bookmark.time.to_date
        when YESTADAY
          result += 2
        when YESTADAY - 1
          result -= 1
        end
      }
      result
    end

    def get_hot_point( entry )
      result = 0
      result -= 11 if @black_list.match( entry.target_url )
      entry.bookmarks.each{|bookmark|
        case bookmark.time.to_date
        when YESTADAY
          result += 3
        when YESTADAY - 1
          result += 2
        when YESTADAY - 2
          result += 1
        end
      }
      result
    end

    def get_chosen_point( entry )
      return -1 if @black_list.match( entry.target_url )
      return -1 if entry.bookmarks.size < CHOSEN_ENTRY_VILLAGERS_THRESHOLD
      result = 0
      entry.bookmarks.each{|bookmark|
        case bookmark.time.to_date
        when YESTADAY
          result += 3
        when YESTADAY - 1
          result -= 1
        when YESTADAY - 2
          result -= 1
        end
      }
      result
    end

    ## -- ᥤ -----
    def start
      ## -- ƼǡѰ
      @database = PStore.new( 'versatile_data.dat' )

      @villagers = {}
      Util.load_village_people_file {|name|
        @villagers[name] = Villager.new( name )
      }
      @collected_daily_data = CollectedDailyData.new( @villagers )
      @black_list = MatchingList.new
      BlackList::FILENAMES.each{|filename|
        @black_list.read_file( Util.filename_to_path( filename ) )
      }

      @daily_template = DailyTemplate.new
      @rss_template = RSSTemplate.new

      @database.transaction{
        @candidate_data = @database.fetch( 'candidate_data', CandidateData.new )
      }

      ## -- 1 ʬǡؿ
      load_cache_or_get_data = lambda{|date|
        daily_data = DailyData.new( date )
        if daily_data.cache_exist?
          daily_data.load_cache
        else
          daily_data.get_data( @villagers )
          daily_data.save_cache
        end
        @collected_daily_data.add_daily_data( daily_data )
      }

      ## -- ѥǡ -----
      YESTADAY.step( YESTADAY - (NEW_ENTRY_COLLECT_DAY - 1), -1 ) {|date|
        load_cache_or_get_data.call( date )
      }

      ## --  1 ʬ html ե
      self.make_new_html

      ## -- ͵ѥǡ
      (YESTADAY - NEW_ENTRY_COLLECT_DAY).step( YESTADAY - (HOT_ENTRY_COLLECT_DAY - 1), -1 ) {|date|
        load_cache_or_get_data.call( date )
      }

      ## -- ͵ 1 ʬ html ե
      self.make_hot_html

      ## -- ԥꥹȤ html ե
      self.make_candidate_html

      ## -- ԥꥹȤ¸
      @database.transaction{
        @database['candidate_data'] = @candidate_data
      }

      ## -- ꤹ html ե
      self.make_chosen_html

      ## -- index.shtml 
      self.make_index_html
    end
  end
end
## -- main ---------------------------------------------------------------
MuraHatebu::TODAY = Date.parse( ARGV[0] ) rescue Date.today
MuraHatebu::YESTADAY = MuraHatebu::TODAY - 1
MuraHatebu::Manager.new.start
