## murahatebu : progress.rb

require 'base.rb'

## вɽ
MuraHatebu::PROGRESS_ENTRY_DISPLAY_COUNT = 100

## вΥե̾
MuraHatebu::PROGRESS_FILENAME = 'progress.html'


module MuraHatebu
  ## -- DailyTemplate ----------------------------------------------------
  class DailyTemplate < HtmlTemplate
    FILENAME = Util.filename_to_path( 'template/daily_template.html' )
    PARAMS_STRUCT = Struct.new( :date, :type, :version )

    OneRowTemplate = define_new_child( :list, [:number, :linked_title, :point, :villager_count] )
    class OneRowTemplate < HtmlTemplate
      FaviconTemplate = define_new_child( :favicons, [:villager, :tags, :comment] )
    end
  end

  ## -- Progress ---------------------------------------------------------
  class Progress
    def get_progress_point( entry )
      return -1 if @black_list.match( entry.target_url )
      # return -1 if entry.bookmarks.size < CHOSEN_ENTRY_VILLAGERS_THRESHOLD
      result = 0
      entry.bookmarks.each{|bookmark|
        case bookmark.time.to_date
        when TODAY
          result += 3
        when TODAY - 1
          result -= 1
        when TODAY - 2
          result -= 1
        end
      }
      result
    end
    
    ## -- в html ե
    def make_progress_html
      entry_count = 0
      @daily_template.reset

      @collected_daily_data.entry_hash.values.sort_by{|entry| [self.get_progress_point( entry ), entry.bookmarks.first.time]
      }.reverse_each{|entry|
        progress_point = self.get_progress_point( entry )

        @daily_template.params.list {|one_row_template|
          one_row_template.params.number = entry_count + 1
          one_row_template.params.linked_title = entry.make_html_linked_title
          one_row_template.params.point = progress_point
          one_row_template.params.villager_count = entry.bookmarks.size
          entry.bookmarks.each{|bookmark|
            one_row_template.params.favicons {|favicon_template|
              favicon_template.params.villager = bookmark.user.name
              favicon_template.params.tags = bookmark.tags.join( ' ' )
              favicon_template.params.comment = bookmark.comment.to_s.gsub( /\'/, "\\\\'" ).gsub( /\"/, '&quot;' )
            }
          }
        }

        entry_count += 1
        break if entry_count >= PROGRESS_ENTRY_DISPLAY_COUNT
      }

      ## -- ƥץ졼Ȥ
      @daily_template.params.date = Util.enc( TODAY.to_japanese_string )
      @daily_template.params.type = Util.enc( 'в ' + Time.now.strftime( '%H:%M:%S' ) )
      @daily_template.params.version = VERSION
      daily_html = @daily_template.make_html
      daily_html.gsub!( /\"\.\.\//, "\"./" )
      Util.file_puts( Util.filename_to_path( PROGRESS_FILENAME ), "w", daily_html )
    end
    
        ## -- ᥤ -----
    def start
      ## -- ƼǡѰ
      @villagers = {}
      Util.load_village_people_file {|name|
        @villagers[name] = Villager.new( name )
      }
      @collected_daily_data = CollectedDailyData.new( @villagers )
      @black_list = MatchingList.new
      BlackList::FILENAMES.each{|filename|
        @black_list.read_file( Util.filename_to_path( filename ) )
      }

      @daily_template = DailyTemplate.new

      ## -- 1 ʬǡؿ
      load_cache_or_get_data = lambda{|date|
        daily_data = DailyData.new( date )
        if daily_data.cache_exist?
          daily_data.load_cache
        else
          daily_data.get_data( @villagers )
          daily_data.save_cache
        end
        @collected_daily_data.add_daily_data( daily_data )
      }

      ## -- ѥǡ -----
      YESTADAY.step( YESTADAY - 1, -1 ) {|date|
        load_cache_or_get_data.call( date )
      }

      today_data = DailyData.new( TODAY )
      today_data.get_data( @villagers )
      @collected_daily_data.add_daily_data( today_data )

      ## -- в html ե
      self.make_progress_html
    end
  end
end

## -- main ---------------------------------------------------------------
MuraHatebu::TODAY = Date.parse( ARGV[0] ) rescue Date.today
MuraHatebu::YESTADAY = MuraHatebu::TODAY - 1
MuraHatebu::Progress.new.start
